@@ -1,3 +1,6 @@
+import re
+import subprocess
+
 import click
 from appollo.helpers import login_required_warning_decorator, ssh_tunnel, print_qrcode, get_version_and_build
 
@@ -157,6 +160,9 @@ def detail(key):
                     Minimum iOS SDK : [bold]{build_instance["min_sdk"]}[/bold]
                     App version: [bold]{build_instance["app_version"] or "-"}[/bold]
                     Build number: [bold]{build_instance["build_number"] or "-"}[/bold]
+                    Mode: [bold]{build_instance["mode"]}[/bold]
+                    Target: [bold]{build_instance["target"] or "-"}[/bold]
+                    Flavor: [bold]{build_instance["flavor"] or "-"}[/bold]
                     """
                 )
             ), title="Build config"))
@@ -270,18 +276,22 @@ def _show_build_progress(ctx, build_instance, tunnel_port=None, tunnel_host=None
 @click.option('--minimal-ios-version', help="Minimal iOS version for you application (example \"9.0\")")
 @click.option('--app-version', help="App version to set for this build (for example \"1.3.1\"). If not set, the version in pubspec.yaml will be used")
 @click.option('--build-number', type=int, help="Build number to set for this build (the number after '+' in the version in pubspec.yaml). If not set, the build number in pubspec.yaml will be used")
+@click.option('--mode', type=click.Choice(["release", "profile", "debug"]), help="Mode to build the app in. Defaults to release")
+@click.option('--target', help="The main entry-point file of the application. Defaults to lib/main.dart")
+@click.option('--flavor', help="Custom app flavor")
 @click.option('--tunnel-port', type=int, help="Start a reverse SSH tunnel when the build is started, forwarding to this port. Note: this only applies to configuration builds")
 @click.option('--tunnel-host', help="If --tunnel-port is specified, this is the host to forward to (defaults to localhost)")
 @click.option('--tunnel-remote-port', type=int, help="If --tunnel-port is specified, this is the port on the VM (defaults to the same port, except for 22 and 5900)")
 @click.option('--no-progress', is_flag=True, help="Do not display the progress and exit the command immediately.")
+@click.option('--no-flutter-warning', is_flag=True, help="Do not display a warning if no flutter version is specified and the local flutter version does not match the build version.")
 @click.pass_context
-def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_number, tunnel_port, tunnel_host, tunnel_remote_port, no_progress, app_key=None, directory=None):
+def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_number, mode, target, flavor, tunnel_port, tunnel_host, tunnel_remote_port, no_progress, no_flutter_warning, app_key=None, directory=None):
     """ Start a new build from scratch
 
-    DIRECTORY : Home directory of the flutter project. If not provided gets the current directory.
+    DIRECTORY : Home directory of the flutter project. If not provided, gets the current directory.
 
     \f
-    The Appollo tool is composed of :
+    The Appollo tool is composed of:
 
         - *Appollo-Remote* : The pre-configured build machines which handle the setup, build and release of your app
         - *Appollo-cli* : The CLI command line that works as an interface to start Appollo-Remote build machines
@@ -298,8 +308,35 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
         * **Publication** : Build an .ipa file and publish it on the App Store. Once this build succeeds you have
           to go on the App Store to complete information and screenshots related to your new application version.
 
-    Killing this command will not stop the build you can check the progress of all your Appollo-Remotes by running
+    Killing this command will not stop the build. You can check the progress of all your Appollo-Remotes by running
     :code:`appollo build ls` or get detailed information by running :code:`appollo build detail` and selecting your build.
+
+    To avoid having to specify all the parameters each time, you can create a .appollo file in the directory where the
+    command is run. Each parameter is specified on a line in the form PARAM=VALUE. For example:
+
+    .. code-block::
+
+       app-key=123
+       flutter=3.0.0
+       minimal-ios-version=11.0
+
+    All files and directories in the provided directory will be uploaded, except:
+
+        * build/
+        * windows/
+        * linux/
+        * .dart_tool/
+        * .pub-cache/
+        * .pub/
+        * .git/
+        * .gradle/
+        * source.zip
+        * .app.zip
+        * appollo.patch
+
+    You can also specify additional files and directories in a .appolloignore file, with each files and directories
+    you want to ignore on separate lines, with directories ending with '/'
+
     """
     import os
     import textwrap
@@ -325,6 +362,8 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
                     print("Error in .appollo file line "+str(i+1)+": should be KEY=VALUE")
                     continue
                 key = split[0].strip()
+                if key[0] == "#":  # Ignore commented lines
+                    continue
                 value = split[1].strip()
                 if key == "app-key":
                     if not app_key:
@@ -344,6 +383,15 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
                 elif key == "build-number":
                     if not build_number:
                         build_number = int(value)
+                elif key == "mode":
+                    if not mode:
+                        mode = value
+                elif key == "target":
+                    if not target:
+                        target = target
+                elif key == "flavor":
+                    if not flavor:
+                        flavor = flavor
                 elif key == "tunnel-port":
                     if not tunnel_port:
                         tunnel_port = int(value)
@@ -356,6 +404,9 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
                 elif key == "no-progress":
                     if no_progress is None:
                         no_progress = value in ["1", "true", "True"]
+                elif key == "no-flutter-warning":
+                    if no_flutter_warning is None:
+                        no_flutter_warning = value in ["1", "true", "True"]
                 else:
                     console.print(f"Warning: unknown option '{key}' in .appollo")
 
@@ -400,13 +451,42 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
             if res not in ["y", "Y"]:
                 return
 
+    # If no flutter version is explicitly specified, check that the local version matches the one of the build so the user doesn't get unexpected errors
+    if not no_flutter_warning and not flutter:
+        flutter_version_output = subprocess.run(["flutter", "--version"], stdout=subprocess.PIPE, text=True)
+        if flutter_version_output.returncode == 0:
+            match = re.match(r"Flutter ([^\s]+) ", flutter_version_output.stdout)
+            if match:
+                local_version = match.group(1)
+                build_version = api.get("/flutter-versions/latest")['version']
+                if local_version.split("-")[0].split(".")[:2] != build_version.split("-")[0].split(".")[:2]:  # Only check major and minor
+                    res = console.input(f"Warning: your local flutter version is {local_version} but the build will be run with the latest flutter version ({build_version}). This could lead to unexpected errors if you have not tested your code with version {build_version}. To avoid this, specify the flutter version you want to use with the --flutter parameter or in a .appollo file.\nWhat do you want to do? (Y/m/c):\nContinue anyway (Y)\nSet the build version to {local_version} (m)\nCancel and specify the version yourself (c)\n")
+                    if res in ["c", "C"]:
+                        return
+                    if res in ["m", "M"]:
+                        flutter = local_version
+
     console.print(f"Zipping {directory}")
-    zip_file = zip_directory(directory)
+    excluded_dirs = []
+    excluded_files = []
+    if os.path.isfile(".appolloignore"):
+        with open(".appolloignore") as ignore:
+            for line in ignore.readlines():
+                line = line.strip()
+                if line == "":
+                    continue
+                if line[-1] == "/":
+                    excluded_dirs.append(line[:-1])
+                else:
+                    excluded_files.append(line)
+
+    zip_file = zip_directory(directory, excluded_dirs, excluded_files)
 
     file_size_mb = round(os.path.getsize(zip_file)/1000000, 2)
 
-    if file_size_mb > 1000:
-        console.print("File size exceeds 1GB, very large applications are not supported by Appollo.")
+    if file_size_mb > 500:
+        console.print("Zipped directory size exceeds 500MB, very large applications are not supported by Appollo. Make sure that all files and directories not needed to build are listed in .appolloignore")
+        os.remove(".app.zip")
         return
 
     console.print(f"Uploading {directory} ({file_size_mb} MB)")
@@ -420,6 +500,9 @@ def start(ctx, build_type, flutter, minimal_ios_version, app_version, build_numb
             "flutter_version": flutter,
             "app_version": app_version,
             "build_number": build_number,
+            "mode": mode,
+            "target": target,
+            "flavor": flavor,
         },
         files={
             "source": ("source.zip", open(".app.zip", "rb"), "application/zip")
